import { __awaiter, __generator, __extends, __assign } from 'tslib';
import { setDOMSize, isBrowser, AbstractRendererPlugin, AbstractRenderer } from '@antv/g-lite';
import * as DeviceRenderer from '@antv/g-plugin-device-renderer';
export { DeviceRenderer };
import * as DomInteraction from '@antv/g-plugin-dom-interaction';
export { DomInteraction };
import * as HTMLRenderer from '@antv/g-plugin-html-renderer';
export { HTMLRenderer };
import * as ImageLoader from '@antv/g-plugin-image-loader';
import * as WebGLDevice from '@antv/g-plugin-webgl-device';
export { WebGLDevice };
import { isString } from '@antv/util';

var WebGLContextService = /** @class */ (function () {
    function WebGLContextService(context) {
        this.canvasConfig = context.config;
        // @ts-ignore
        this.deviceRendererPlugin = context.deviceRendererPlugin;
    }
    WebGLContextService.prototype.init = function () {
        var _a = this.canvasConfig, container = _a.container, canvas = _a.canvas;
        if (canvas) {
            this.$canvas = canvas;
            if (container &&
                canvas.parentElement !== container) {
                container.appendChild(canvas);
            }
            this.$container = canvas.parentElement;
            this.canvasConfig.container = this.$container;
        }
        else if (container) {
            // create container
            this.$container = isString(container)
                ? document.getElementById(container)
                : container;
            if (this.$container) {
                // create canvas
                var $canvas = document.createElement('canvas');
                this.$container.appendChild($canvas);
                if (!this.$container.style.position) {
                    this.$container.style.position = 'relative';
                }
                this.$canvas = $canvas;
            }
        }
        this.resize(this.canvasConfig.width, this.canvasConfig.height);
    };
    WebGLContextService.prototype.getDomElement = function () {
        return this.$canvas;
    };
    WebGLContextService.prototype.getContext = function () {
        // @ts-ignore
        return this.deviceRendererPlugin.getDevice().gl;
    };
    WebGLContextService.prototype.getBoundingClientRect = function () {
        if (this.$canvas.getBoundingClientRect) {
            return this.$canvas.getBoundingClientRect();
        }
    };
    WebGLContextService.prototype.destroy = function () {
        // @ts-ignore
        if (this.$container && this.$canvas && this.$canvas.parentNode) {
            // @ts-ignore
            this.$container.removeChild(this.$canvas);
        }
    };
    WebGLContextService.prototype.resize = function (width, height) {
        // use user-defined dpr first
        var devicePixelRatio = this.canvasConfig.devicePixelRatio;
        var dpr = devicePixelRatio || (isBrowser && window.devicePixelRatio) || 1;
        dpr = dpr >= 1 ? Math.ceil(dpr) : 1;
        this.dpr = dpr;
        if (this.$canvas) {
            var dpr_1 = this.getDPR();
            // set canvas width & height
            this.$canvas.width = dpr_1 * width;
            this.$canvas.height = dpr_1 * height;
            // set CSS style width & height
            setDOMSize(this.$canvas, width, height);
        }
    };
    WebGLContextService.prototype.getDPR = function () {
        return this.dpr;
    };
    WebGLContextService.prototype.applyCursorStyle = function (cursor) {
        if (this.$container && this.$container.style) {
            this.$container.style.cursor = cursor;
        }
    };
    WebGLContextService.prototype.toDataURL = function (options) {
        if (options === void 0) { options = {}; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                // @see https://stackoverflow.com/questions/26783586/canvas-todataurl-returns-blank-image
                return [2 /*return*/, this.deviceRendererPlugin.toDataURL(options)];
            });
        });
    };
    return WebGLContextService;
}());

var ContextRegisterPlugin = /** @class */ (function (_super) {
    __extends(ContextRegisterPlugin, _super);
    function ContextRegisterPlugin(rendererPlugin) {
        var _this = _super.call(this) || this;
        _this.rendererPlugin = rendererPlugin;
        _this.name = 'webgl-context-register';
        return _this;
    }
    ContextRegisterPlugin.prototype.init = function () {
        this.context.ContextService = WebGLContextService;
        // @ts-ignore
        this.context.deviceRendererPlugin = this.rendererPlugin;
    };
    ContextRegisterPlugin.prototype.destroy = function () {
        delete this.context.ContextService;
    };
    return ContextRegisterPlugin;
}(AbstractRendererPlugin));

var Renderer = /** @class */ (function (_super) {
    __extends(Renderer, _super);
    function Renderer(config) {
        var _this = _super.call(this, config) || this;
        var deviceRendererPlugin = new DeviceRenderer.Plugin();
        _this.registerPlugin(new ContextRegisterPlugin(deviceRendererPlugin));
        _this.registerPlugin(new ImageLoader.Plugin());
        _this.registerPlugin(new WebGLDevice.Plugin(__assign(__assign({}, ((config === null || config === void 0 ? void 0 : config.targets)
            ? {
                targets: config.targets,
            }
            : {
                targets: ['webgl2', 'webgl1'],
            })), { onContextLost: config === null || config === void 0 ? void 0 : config.onContextLost, onContextRestored: config === null || config === void 0 ? void 0 : config.onContextRestored, onContextCreationError: config === null || config === void 0 ? void 0 : config.onContextCreationError })));
        _this.registerPlugin(deviceRendererPlugin);
        _this.registerPlugin(new DomInteraction.Plugin());
        _this.registerPlugin(new HTMLRenderer.Plugin());
        return _this;
    }
    return Renderer;
}(AbstractRenderer));

export { Renderer };
//# sourceMappingURL=index.esm.js.map
